@@ -19,14 +19,13 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <string.h>
 #include <limits.h>
 #include <errno.h>
 
 #include <gmodule.h>
-#include <glib/gstdio.h>
 
 #include "pango-enum-types.h"
 #include "pango-modules.h"
@@ -138,7 +137,7 @@ pango_find_map (PangoLanguage *language,
 
   if (!tmp_list)
     {
-      map_info = g_slice_new (PangoMapInfo);
+      map_info = g_new (PangoMapInfo, 1);
       map_info->language = language;
       map_info->engine_type_id = engine_type_id;
       map_info->render_type_id = render_type_id;
@@ -169,10 +168,10 @@ pango_module_load (GTypeModule *module)
 
   if (pango_module->path)
     {
-      pango_module->library = g_module_open (pango_module->path, G_MODULE_BIND_LAZY | G_MODULE_BIND_LOCAL);
+      pango_module->library = g_module_open (pango_module->path, 0);
       if (!pango_module->library)
 	{
-	  g_warning ("%s", g_module_error());
+	  g_warning (g_module_error());
 	  return FALSE;
 	}
       
@@ -186,7 +185,7 @@ pango_module_load (GTypeModule *module)
 	  !g_module_symbol (pango_module->library, "script_engine_create", 
 			    (gpointer *)&pango_module->create))
 	{
-	  g_warning ("%s", g_module_error());
+	  g_warning (g_module_error());
 	  g_module_close (pango_module->library);
 	  
 	  return FALSE;
@@ -286,7 +285,7 @@ handle_included_module (PangoIncludedModule *included_module,
 
   for (i = 0; i < n_engines; i++)
     {
-      PangoEnginePair *pair = g_slice_new (PangoEnginePair);
+      PangoEnginePair *pair = g_new (PangoEnginePair, 1);
 
       pair->info = engine_info[i];
       pair->module = module;
@@ -354,12 +353,6 @@ script_from_string (const char *str)
   return value->value;
 }
 
-static void
-script_info_free (PangoEngineScriptInfo *script_info, gpointer data)
-{
-  g_slice_free (PangoEngineScriptInfo, script_info);
-}
-
 static gboolean /* Returns true if succeeded, false if failed */
 process_module_file (FILE *module_file)
 {
@@ -369,7 +362,7 @@ process_module_file (FILE *module_file)
 
   while (pango_read_line (module_file, line_buf))
     {
-      PangoEnginePair *pair = g_slice_new (PangoEnginePair);
+      PangoEnginePair *pair = g_new (PangoEnginePair, 1);
       PangoEngineScriptInfo *script_info;
       PangoScript script;
       GList *scripts = NULL;
@@ -383,7 +376,7 @@ process_module_file (FILE *module_file)
 
       if (!pango_skip_space (&p))
 	{
-	  g_slice_free (PangoEnginePair, pair);
+	  g_free (pair);
 	  continue;
 	}
 
@@ -425,7 +418,7 @@ process_module_file (FILE *module_file)
 		  goto error;
 		}
 	      
-	      script_info = g_slice_new (PangoEngineScriptInfo);
+	      script_info = g_new (PangoEngineScriptInfo, 1);
 	      script_info->script = script;
 	      script_info->langs = g_strdup (q + 1);
 	      
@@ -460,13 +453,13 @@ process_module_file (FILE *module_file)
       dlloaded_engines = g_slist_prepend (dlloaded_engines, pair);
 
     error:
-      g_list_foreach (scripts, (GFunc)script_info_free, NULL);
+      g_list_foreach (scripts, (GFunc)g_free, NULL);
       g_list_free (scripts);
 
       if (have_error)
 	{
 	  g_printerr ("Error reading Pango modules file\n");
-	  g_slice_free(PangoEnginePair, pair);
+	  g_free(pair);
 	  break;
 	}
     }
@@ -501,7 +494,7 @@ read_modules (void)
 
   while (n-- > 0)
     {
-      module_file = g_fopen (files[n], "r");
+      module_file = fopen (files[n], "r");
       if (module_file)
 	{
 	  process_module_file(module_file);
@@ -551,7 +544,7 @@ map_add_engine (PangoMapInfo    *info,
 	}
 
       script = pair->info.scripts[i].script;
-      if ((guint)script >= map->entries->len)
+      if (script >= map->entries->len)
 	g_array_set_size (map->entries, script + 1);
 
       entry = &g_array_index (map->entries, PangoMapEntry, script);
@@ -587,6 +580,8 @@ map_add_engine_list (PangoMapInfo *info,
 static void
 build_map (PangoMapInfo *info)
 {
+  PangoMap *map;
+
   const char *engine_type = g_quark_to_string (info->engine_type_id);
   const char *render_type = g_quark_to_string (info->render_type_id);
   
@@ -600,11 +595,11 @@ build_map (PangoMapInfo *info)
 	  gchar *filename = g_build_filename (pango_get_sysconf_subdirectory (),
 					      "pango.modules",
 					      NULL);
-	  g_warning ("No builtin or dynamically\n"
-		     "loaded modules were found. Pango will not work correctly.\n"
-		     "This probably means there was an error in the creation of:\n"
+	  g_warning ("No builtin or dynamically loaded modules\n"
+		     "were found. Pango will not work correctly. This probably means\n"
+		     "there was an error in the creation of:\n"
 		     "  '%s'\n"
-		     "You should create this file by running pango-querymodules.",
+		     "You may be able to recreate this file by running pango-querymodules.",
 		     filename);
 	  g_free (filename);
 	  
@@ -612,8 +607,8 @@ build_map (PangoMapInfo *info)
 	}
     }
   
-  info->map = g_slice_new (PangoMap);
-  info->map->entries = g_array_new (FALSE, TRUE, sizeof (PangoMapEntry));
+  info->map = map = g_new (PangoMap, 1);
+  map->entries = g_array_new (FALSE, TRUE, sizeof (PangoMapEntry));
 			      
   map_add_engine_list (info, dlloaded_engines, engine_type, render_type);  
   map_add_engine_list (info, registered_engines, engine_type, render_type);  
@@ -639,7 +634,7 @@ pango_map_get_engine (PangoMap   *map,
   PangoMapEntry *entry = NULL;
   PangoMapEntry *common_entry = NULL;
   
-  if ((guint)script < map->entries->len)
+  if (script < map->entries->len)
     entry = &g_array_index (map->entries, PangoMapEntry, script);
 
   if (PANGO_SCRIPT_COMMON < map->entries->len)
@@ -657,7 +652,7 @@ pango_map_get_engine (PangoMap   *map,
     return NULL;
 }
 
-static void
+void
 append_engines (GSList **engine_list,
 		GSList  *pair_list)
 {
@@ -696,7 +691,7 @@ pango_map_get_engines (PangoMap     *map,
   PangoMapEntry *entry = NULL;
   PangoMapEntry *common_entry = NULL;
   
-  if ((guint)script < map->entries->len)
+  if (script < map->entries->len)
     entry = &g_array_index (map->entries, PangoMapEntry, script);
 
   if (PANGO_SCRIPT_COMMON < map->entries->len)
