@@ -150,6 +150,9 @@ char *                pango_font_description_to_filename (const PangoFontDescrip
 /*
  * PangoFontMetrics
  */
+
+#define PANGO_TYPE_FONT_METRICS  (pango_font_metrics_get_type ())
+GType             pango_font_metrics_get_type                    (void);
 PangoFontMetrics *pango_font_metrics_ref                         (PangoFontMetrics *metrics);
 void              pango_font_metrics_unref                       (PangoFontMetrics *metrics);
 int               pango_font_metrics_get_ascent                  (PangoFontMetrics *metrics);
@@ -190,6 +193,7 @@ void                 pango_font_family_list_faces (PangoFontFamily  *family,
 						   PangoFontFace  ***faces,
 						   int              *n_faces);
 G_CONST_RETURN char *pango_font_family_get_name   (PangoFontFamily  *family);
+gboolean   pango_font_family_is_monospace         (PangoFontFamily  *family);
 
 #ifdef PANGO_ENABLE_BACKEND
 
@@ -207,11 +211,21 @@ struct _PangoFontFamily
 struct _PangoFontFamilyClass
 {
   GObjectClass parent_class;
+
+  /*< public >*/
   
   void  (*list_faces)      (PangoFontFamily  *family,
 		            PangoFontFace  ***faces,
 		            int              *n_faces);
   const char * (*get_name) (PangoFontFamily  *family);
+  gboolean (*is_monospace) (PangoFontFamily *family);
+
+  /*< private >*/
+
+  /* Padding for future expansion */
+  void (*_pango_reserved2) (void);
+  void (*_pango_reserved3) (void);
+  void (*_pango_reserved4) (void);
 };
 
 #endif /* PANGO_ENABLE_BACKEND */
@@ -228,6 +242,9 @@ GType      pango_font_face_get_type       (void) G_GNUC_CONST;
 
 PangoFontDescription *pango_font_face_describe       (PangoFontFace *face);
 G_CONST_RETURN char  *pango_font_face_get_face_name (PangoFontFace *face);
+void                  pango_font_face_list_sizes     (PangoFontFace  *face, 
+                                                      int           **sizes, 
+                                                      int            *n_sizes);
 
 #ifdef PANGO_ENABLE_BACKEND
 
@@ -245,9 +262,21 @@ struct _PangoFontFace
 struct _PangoFontFaceClass
 {
   GObjectClass parent_class;
+
+  /*< public >*/
   
   const char           * (*get_face_name) (PangoFontFace *face);
   PangoFontDescription * (*describe)       (PangoFontFace *face);
+  void                   (*list_sizes)     (PangoFontFace  *face, 
+                                            int           **sizes, 
+                                            int            *n_sizes);
+
+  /*< private >*/
+
+  /* Padding for future expansion */
+  void (*_pango_reserved2) (void);
+  void (*_pango_reserved3) (void);
+  void (*_pango_reserved4) (void);
 };
 
 #endif /* PANGO_ENABLE_BACKEND */
@@ -291,6 +320,8 @@ struct _PangoFont
 struct _PangoFontClass
 {
   GObjectClass parent_class;
+
+  /*< public >*/
   
   PangoFontDescription *(*describe)           (PangoFont      *font);
   PangoCoverage *       (*get_coverage)       (PangoFont      *font,
@@ -304,6 +335,13 @@ struct _PangoFontClass
 					       PangoRectangle *logical_rect);
   PangoFontMetrics *    (*get_metrics)        (PangoFont      *font,
 					       PangoLanguage  *language);
+  /*< private >*/
+
+  /* Padding for future expansion */
+  void (*_pango_reserved1) (void);
+  void (*_pango_reserved2) (void);
+  void (*_pango_reserved3) (void);
+  void (*_pango_reserved4) (void);
 };
 
 #endif /* PANGO_ENABLE_BACKEND */
