@@ -75,6 +75,8 @@ struct _PangoWin32Family
 
   char *family_name;
   GSList *font_entries;
+
+  gboolean is_monospace;
 };
 
 struct _PangoWin32SizeInfo
@@ -93,6 +95,10 @@ struct _PangoWin32SizeInfo
 GType             pango_win32_family_get_type        (void);
 GType             pango_win32_face_get_type          (void);
 
+static void       pango_win32_face_list_sizes        (PangoFontFace  *face,
+                                                      int           **sizes,
+                                                      int            *n_sizes);
+
 static void       pango_win32_font_map_init          (PangoWin32FontMap       *fontmap);
 static void       pango_win32_font_map_class_init    (PangoFontMapClass       *class);
 
@@ -187,7 +193,8 @@ pango_win32_font_map_class_init (PangoFontMapClass *class)
   object_class->finalize = pango_win32_font_map_finalize;
   class->load_font = pango_win32_font_map_load_font;
   class->list_families = pango_win32_font_map_list_families;
-
+  class->shape_engine_type = PANGO_RENDER_TYPE_WIN32;
+  
   pango_win32_get_dc ();
 }
 
@@ -316,18 +323,27 @@ pango_win32_family_list_faces (PangoFontFamily  *family,
     }
 }
 
-const char *
+static const char *
 pango_win32_family_get_name (PangoFontFamily  *family)
 {
   PangoWin32Family *win32family = PANGO_WIN32_FAMILY (family);
   return win32family->family_name;
 }
 
+static gboolean
+pango_win32_family_is_monospace (PangoFontFamily *family)
+{
+  PangoWin32Family *win32family = PANGO_WIN32_FAMILY (family);
+
+  return win32family->is_monospace;
+}
+
 static void
 pango_win32_family_class_init (PangoFontFamilyClass *class)
 {
   class->list_faces = pango_win32_family_list_faces;
   class->get_name = pango_win32_family_get_name;
+  class->is_monospace = pango_win32_family_is_monospace;
 }
 
 GType
@@ -785,6 +801,7 @@ pango_win32_insert_font (PangoWin32FontMap *win32fontmap,
     { 
     case FF_MODERN : /* monospace */
       PING(("monospace"));
+      font_family->is_monospace = TRUE; /* modify before reuse */
       font_family = pango_win32_get_font_family (win32fontmap, "monospace");
       font_family->font_entries = g_slist_append (font_family->font_entries, win32face);
       win32fontmap->n_fonts++;
@@ -932,6 +949,20 @@ pango_win32_face_class_init (PangoFontFaceClass *class)
 {
   class->describe = pango_win32_face_describe;
   class->get_face_name = pango_win32_face_get_face_name;
+  class->list_sizes = pango_win32_face_list_sizes;
+}
+
+static void
+pango_win32_face_list_sizes (PangoFontFace  *face,
+                             int           **sizes,
+                             int            *n_sizes)
+{
+  /*
+   * for scalable fonts it's simple, and currently we only have such
+   * see : pango_win32_enum_proc(), TRUETYPE_FONTTYPE
+   */
+  *sizes = NULL;
+  *n_sizes = 0;
 }
 
 GType
